@@ -126,6 +126,7 @@ typedef struct
   EmpathyDispatcher *dispatcher;
   EmpathyDispatchOperation *operation;
   TpConnection *connection;
+  gboolean should_ensure;
   gchar *channel_type;
   guint handle_type;
   guint handle;
@@ -180,7 +181,8 @@ typedef struct
 } FindChannelRequest;
 
 static void
-empathy_dispatcher_call_create_channel (EmpathyDispatcher *dispatcher,
+empathy_dispatcher_call_create_or_ensure_channel (
+    EmpathyDispatcher *dispatcher,
     DispatcherRequestData *request_data);
 
 static DispatchData *
@@ -221,6 +223,8 @@ new_dispatcher_request_data (EmpathyDispatcher *dispatcher,
   result->dispatcher = g_object_ref (dispatcher);
   result->connection = connection;
 
+  result->should_ensure = FALSE;
+
   result->channel_type = g_strdup (channel_type);
   result->handle_type = handle_type;
   result->handle = handle;
@@ -744,6 +748,7 @@ dispatcher_connection_got_all (TpProxy *proxy,
   EmpathyDispatcher *dispatcher = EMPATHY_DISPATCHER (object);
   EmpathyDispatcherPriv *priv = GET_PRIV (dispatcher);
   GPtrArray *requestable_channels;
+  GPtrArray *existing_channels;
 
   if (error) {
     DEBUG ("Error: %s", error->message);
@@ -788,6 +793,32 @@ dispatcher_connection_got_all (TpProxy *proxy,
 
       g_hash_table_remove (priv->outstanding_classes_requests, proxy);
     }
+
+  existing_channels = tp_asv_get_boxed (properties,
+      "Channels", TP_ARRAY_TYPE_CHANNEL_DETAILS_LIST);
+
+  if (existing_channels != NULL)
+    {
+      int idx;
+
+      for (idx = 0; idx < existing_channels->len; idx++)
+        {
+          GValueArray *values = g_ptr_array_index (existing_channels, idx);
+          const gchar *object_path;
+          GHashTable *properties;
+
+          object_path = g_value_get_boxed (g_value_array_get_nth (values, 0));
+          properties = g_value_get_boxed (g_value_array_get_nth (values, 1));
+
+          if (tp_strdiff (tp_asv_get_string (properties,
+                      TP_IFACE_CHANNEL ".ChannelType"),
+                  TP_IFACE_CHANNEL_TYPE_TEXT))
+            continue;
+
+          dispatcher_connection_new_channel_with_properties (dispatcher,
+              TP_CONNECTION (proxy), object_path, properties);
+        }
+    }
 }
 
 static void
@@ -840,7 +871,9 @@ dispatcher_init_connection_if_needed (EmpathyDispatcher *dispatcher,
         1, TP_CHANNEL_MEDIA_CAPABILITY_AUDIO |
            TP_CHANNEL_MEDIA_CAPABILITY_VIDEO |
            TP_CHANNEL_MEDIA_CAPABILITY_NAT_TRAVERSAL_STUN  |
-           TP_CHANNEL_MEDIA_CAPABILITY_NAT_TRAVERSAL_GTALK_P2P, G_MAXUINT);
+           TP_CHANNEL_MEDIA_CAPABILITY_NAT_TRAVERSAL_GTALK_P2P |
+           TP_CHANNEL_MEDIA_CAPABILITY_NAT_TRAVERSAL_ICE_UDP,
+           G_MAXUINT);
   g_ptr_array_add (capabilities, g_value_get_boxed (&cap));
 
   tp_cli_connection_interface_capabilities_call_advertise_capabilities (
@@ -1091,10 +1124,12 @@ empathy_dispatcher_init (EmpathyDispatcher *dispatcher)
 
   priv->channels = NULL;
 
-  connections = empathy_account_manager_dup_connections (priv->account_manager);
+  connections = empathy_account_manager_dup_connections (
+      priv->account_manager);
   for (l = connections; l; l = l->next)
     {
-      dispatcher_new_connection_cb (priv->account_manager, l->data, dispatcher);
+      dispatcher_new_connection_cb (priv->account_manager, l->data,
+          dispatcher);
       g_object_unref (l->data);
     }
   g_list_free (connections);
@@ -1142,17 +1177,22 @@ dispatcher_request_failed (EmpathyDispatcher *dispatcher,
   if (request_data->cb != NULL)
     request_data->cb (NULL, error, request_data->user_data);
 
-  conn_data->outstanding_requests =
-      g_list_remove (conn_data->outstanding_requests, request_data);
+  if (conn_data != NULL)
+    {
+      conn_data->outstanding_requests =
+          g_list_remove (conn_data->outstanding_requests, request_data);
+    }
+  /* else Connection has been invalidated */
+
   free_dispatcher_request_data (request_data);
 }
 
 static void
 dispatcher_connection_new_requested_channel (EmpathyDispatcher *dispatcher,
-                                             DispatcherRequestData *request_data,
-                                             const gchar *object_path,
-                                             GHashTable *properties,
-                                             const GError *error)
+  DispatcherRequestData *request_data,
+  const gchar *object_path,
+  GHashTable *properties,
+  const GError *error)
 {
   EmpathyDispatcherPriv *priv = GET_PRIV (dispatcher);
   EmpathyDispatchOperation *operation = NULL;
@@ -1277,8 +1317,8 @@ dispatcher_request_channel (DispatcherRequestData *request_data)
         tp_asv_set_uint32 (request_data->request,
           TP_IFACE_CHANNEL ".TargetHandle", request_data->handle);
 
-      empathy_dispatcher_call_create_channel (request_data->dispatcher,
-        request_data);
+      empathy_dispatcher_call_create_or_ensure_channel (
+        request_data->dispatcher, request_data);
     }
   else
     {
@@ -1311,9 +1351,10 @@ empathy_dispatcher_chat_with_contact (EmpathyContact *contact,
   connection_data = g_hash_table_lookup (priv->connections, connection);
 
   /* The contact handle might not be known yet */
-  request_data  = new_dispatcher_request_data (dispatcher, connection,
+  request_data = new_dispatcher_request_data (dispatcher, connection,
     TP_IFACE_CHANNEL_TYPE_TEXT, TP_HANDLE_TYPE_CONTACT,
     empathy_contact_get_handle (contact), NULL, contact, callback, user_data);
+  request_data->should_ensure = TRUE;
 
   connection_data->outstanding_requests = g_list_prepend
     (connection_data->outstanding_requests, request_data);
@@ -1464,13 +1505,40 @@ dispatcher_create_channel_cb (TpConnection *connect,
 }
 
 static void
-empathy_dispatcher_call_create_channel (EmpathyDispatcher *dispatcher,
+dispatcher_ensure_channel_cb (TpConnection *connect,
+                              gboolean is_ours,
+                              const gchar *object_path,
+                              GHashTable *properties,
+                              const GError *error,
+                              gpointer user_data,
+                              GObject *weak_object)
+{
+  EmpathyDispatcher *dispatcher = EMPATHY_DISPATCHER (weak_object);
+  DispatcherRequestData *request_data = (DispatcherRequestData *) user_data;
+
+  dispatcher_connection_new_requested_channel (dispatcher,
+    request_data, object_path, properties, error);
+}
+
+static void
+empathy_dispatcher_call_create_or_ensure_channel (
+    EmpathyDispatcher *dispatcher,
     DispatcherRequestData *request_data)
 {
-  tp_cli_connection_interface_requests_call_create_channel (
-    request_data->connection, -1,
-    request_data->request, dispatcher_create_channel_cb, request_data, NULL,
-    G_OBJECT (request_data->dispatcher));
+  if (request_data->should_ensure)
+    {
+      tp_cli_connection_interface_requests_call_ensure_channel (
+          request_data->connection, -1,
+          request_data->request, dispatcher_ensure_channel_cb,
+          request_data, NULL, G_OBJECT (request_data->dispatcher));
+    }
+  else
+    {
+      tp_cli_connection_interface_requests_call_create_channel (
+          request_data->connection, -1,
+          request_data->request, dispatcher_create_channel_cb,
+          request_data, NULL, G_OBJECT (request_data->dispatcher));
+    }
 }
 
 void
@@ -1511,7 +1579,7 @@ empathy_dispatcher_create_channel (EmpathyDispatcher *dispatcher,
   connection_data->outstanding_requests = g_list_prepend
     (connection_data->outstanding_requests, request_data);
 
-  empathy_dispatcher_call_create_channel (dispatcher, request_data);
+  empathy_dispatcher_call_create_or_ensure_channel (dispatcher, request_data);
 }
 
 static gboolean
