@@ -11,8 +11,9 @@ G_BEGIN_DECLS
 #include <libempathy/empathy-contact.h>
 #define EMPATHY_TYPE_CAPABILITIES empathy_capabilities_get_type()
 GType empathy_capabilities_get_type (void);
-#define EMPATHY_TYPE_CONTACT_READY empathy_contact_ready_get_type()
-GType empathy_contact_ready_get_type (void);
+#include <libempathy/empathy-contact-list.h>
+#define EMPATHY_TYPE_CONTACT_LIST_FLAGS empathy_contact_list_flags_get_type()
+GType empathy_contact_list_flags_get_type (void);
 #include <libempathy/empathy-debug.h>
 #define EMPATHY_TYPE_DEBUG_FLAGS empathy_debug_flags_get_type()
 GType empathy_debug_flags_get_type (void);
@@ -22,6 +23,9 @@ GType empathy_dispatch_operation_state_get_type (void);
 #include <libempathy/empathy-tp-call.h>
 #define EMPATHY_TYPE_TP_CALL_STATUS empathy_tp_call_status_get_type()
 GType empathy_tp_call_status_get_type (void);
+#include <libempathy/empathy-tp-file.h>
+#define EMPATHY_TYPE_FT_ERROR_ENUM empathy_ft_error_enum_get_type()
+GType empathy_ft_error_enum_get_type (void);
 G_END_DECLS
 
 #endif /* __LIBEMPATHY_ENUM_TYPES_H__ */
